from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit

MKMapSnapshotCompletionHandler = b"v@@"


class TestMKMapSnapshotter(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(MapKit.MKMapSnapshotter, objc.objc_class)

        self.assertResultIsBOOL(MapKit.MKMapSnapshotter.isLoading)
        # self.assertArgIsBOOL(MapKit.MKMapSnapshotter.setLoading_, 0)

        self.assertArgIsBlock(
            MapKit.MKMapSnapshotter.startWithCompletionHandler_,
            0,
            MKMapSnapshotCompletionHandler,
        )

        # XXX: Argument 0 is a dispatch queue, not wrapped...
        self.assertArgIsBlock(
            MapKit.MKMapSnapshotter.startWithQueue_completionHandler_,
            1,
            MKMapSnapshotCompletionHandler,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level

import MapKit


class TestMKDirectionsTypes(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(MapKit.MKDirectionsTransportType)

    @min_os_level("10.9")
    def testConstants(self):
        self.assertEqual(MapKit.MKDirectionsTransportTypeAutomobile, 1 << 0)
        self.assertEqual(MapKit.MKDirectionsTransportTypeWalking, 1 << 1)
        self.assertEqual(MapKit.MKDirectionsTransportTypeAny, 0x0FFFFFFF)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(MapKit.MKDirectionsTransportTypeTransit, 1 << 2)



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit


class TestMKMapSnapshotOptions(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(MapKit.MKMapSnapshotOptions, objc.objc_class)

        self.assertResultIsBOOL(MapKit.MKMapSnapshotOptions.showsPointsOfInterest)
        self.assertArgIsBOOL(MapKit.MKMapSnapshotOptions.setShowsPointsOfInterest_, 0)

        self.assertResultIsBOOL(MapKit.MKMapSnapshotOptions.showsBuildings)
        self.assertArgIsBOOL(MapKit.MKMapSnapshotOptions.setShowsBuildings_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit


class TestMKAnnotationView(TestCase):
    def test_typed_enum(self):
        self.assertIsTypedEnum(MapKit.MKAnnotationViewZPriority, float)
        self.assertIsTypedEnum(MapKit.MKFeatureDisplayPriority, float)

    def test_enum_types(self):
        self.assertIsEnumType(MapKit.MKAnnotationViewCollisionMode)
        self.assertIsEnumType(MapKit.MKAnnotationViewDragState)

    @min_os_level("10.9")
    def testConstants(self):
        self.assertIsInstance(MapKit.MKAnnotationCalloutInfoDidChangeNotification, str)

        self.assertEqual(MapKit.MKAnnotationViewDragStateNone, 0)
        self.assertEqual(MapKit.MKAnnotationViewDragStateStarting, 1)
        self.assertEqual(MapKit.MKAnnotationViewDragStateDragging, 2)
        self.assertEqual(MapKit.MKAnnotationViewDragStateCanceling, 3)
        self.assertEqual(MapKit.MKAnnotationViewDragStateEnding, 4)

        self.assertEqual(MapKit.MKFeatureDisplayPriorityRequired, 1000)
        self.assertEqual(MapKit.MKFeatureDisplayPriorityDefaultHigh, 750)
        self.assertEqual(MapKit.MKFeatureDisplayPriorityDefaultLow, 250)

        self.assertEqual(MapKit.MKAnnotationViewCollisionModeRectangle, 0)
        self.assertEqual(MapKit.MKAnnotationViewCollisionModeCircle, 1)
        self.assertEqual(MapKit.MKAnnotationViewCollisionModeNone, 2)

        self.assertEqual(MapKit.MKAnnotationViewZPriorityMax, 1000.0)
        self.assertEqual(MapKit.MKAnnotationViewZPriorityDefaultSelected, 1000.0)
        self.assertEqual(MapKit.MKAnnotationViewZPriorityDefaultUnselected, 500.0)
        self.assertEqual(MapKit.MKAnnotationViewZPriorityMin, 0.0)

    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(MapKit.MKAnnotationView, objc.objc_class)

    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultHasType(
            MapKit.MKAnnotationView.centerOffset, MapKit.CGPoint.__typestr__
        )
        self.assertArgHasType(
            MapKit.MKAnnotationView.setCenterOffset_, 0, MapKit.CGPoint.__typestr__
        )

        self.assertResultHasType(
            MapKit.MKAnnotationView.calloutOffset, MapKit.CGPoint.__typestr__
        )
        self.assertArgHasType(
            MapKit.MKAnnotationView.setCalloutOffset_, 0, MapKit.CGPoint.__typestr__
        )

        self.assertResultHasType(
            MapKit.MKAnnotationView.leftCalloutOffset, MapKit.CGPoint.__typestr__
        )
        self.assertArgHasType(
            MapKit.MKAnnotationView.setLeftCalloutOffset_, 0, MapKit.CGPoint.__typestr__
        )

        self.assertResultHasType(
            MapKit.MKAnnotationView.rightCalloutOffset, MapKit.CGPoint.__typestr__
        )
        self.assertArgHasType(
            MapKit.MKAnnotationView.setRightCalloutOffset_,
            0,
            MapKit.CGPoint.__typestr__,
        )

        self.assertResultIsBOOL(MapKit.MKAnnotationView.isEnabled)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setEnabled_, 0)

        self.assertResultIsBOOL(MapKit.MKAnnotationView.isHighlighted)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setHighlighted_, 0)

        self.assertResultIsBOOL(MapKit.MKAnnotationView.isSelected)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setSelected_, 0)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setSelected_animated_, 0)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setSelected_animated_, 1)

        self.assertResultIsBOOL(MapKit.MKAnnotationView.canShowCallout)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setCanShowCallout_, 0)

        self.assertResultIsBOOL(MapKit.MKAnnotationView.isDraggable)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setDraggable_, 0)
        self.assertArgIsBOOL(MapKit.MKAnnotationView.setDragState_animated_, 1)



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit


class TestMKGeometry(TestCase):
    @min_os_level("10.9")
    def testStructs(self):
        s = MapKit.MKCoordinateSpan()
        self.assertIsInstance(s.latitudeDelta, float)
        self.assertIsInstance(s.longitudeDelta, float)
        self.assertPickleRoundTrips(s)

        s = MapKit.MKCoordinateRegion()
        self.assertIsInstance(s.center, MapKit.CLLocationCoordinate2D)
        self.assertIsInstance(s.span, MapKit.MKCoordinateSpan)
        self.assertPickleRoundTrips(s)

        s = MapKit.MKMapPoint()
        self.assertIsInstance(s.x, float)
        self.assertIsInstance(s.y, float)
        self.assertPickleRoundTrips(s)

        s = MapKit.MKMapSize()
        self.assertIsInstance(s.width, float)
        self.assertIsInstance(s.height, float)
        self.assertPickleRoundTrips(s)

        s = MapKit.MKMapRect()
        self.assertIsInstance(s.origin, MapKit.MKMapPoint)
        self.assertIsInstance(s.size, MapKit.MKMapSize)
        self.assertPickleRoundTrips(s)

    @min_os_level("10.9")
    def testConstants(self):
        self.assertIsInstance(MapKit.MKMapSizeWorld, MapKit.MKMapSize)
        self.assertIsInstance(MapKit.MKMapRectWorld, MapKit.MKMapRect)
        self.assertIsInstance(MapKit.MKMapRectNull, MapKit.MKMapRect)

    @min_os_level("10.9")
    def test_functions(self):
        c = MapKit.MKCoordinateSpanMake(0.5, 2.5)
        self.assertIsInstance(c, MapKit.MKCoordinateSpan)
        self.assertEqual(c, MapKit.MKCoordinateSpan(0.5, 2.5))
        self.assertEqual(c.latitudeDelta, 0.5)
        self.assertEqual(c.longitudeDelta, 2.5)

        c = MapKit.MKCoordinateRegionMake(
            MapKit.CLLocationCoordinate2D(), MapKit.MKCoordinateSpan()
        )
        self.assertIsInstance(c, MapKit.MKCoordinateRegion)
        self.assertIsInstance(c.center, MapKit.CLLocationCoordinate2D)
        self.assertIsInstance(c.span, MapKit.MKCoordinateSpan)

        c = MapKit.MKCoordinateRegionMakeWithDistance(
            MapKit.CLLocationCoordinate2D(), 1000, 1500
        )
        self.assertIsInstance(c, MapKit.MKCoordinateRegion)
        self.assertIsInstance(c.center, MapKit.CLLocationCoordinate2D)
        self.assertIsInstance(c.span, MapKit.MKCoordinateSpan)

        c = MapKit.MKMapPointForCoordinate(MapKit.CLLocationCoordinate2D())
        self.assertIsInstance(c, MapKit.MKMapPoint)

        c = MapKit.MKCoordinateForMapPoint(MapKit.MKMapPoint())
        self.assertIsInstance(c, MapKit.CLLocationCoordinate2D)

        self.assertArgHasType(MapKit.MKMetersPerMapPointAtLatitude, 0, objc._C_DBL)
        self.assertResultHasType(MapKit.MKMetersPerMapPointAtLatitude, objc._C_DBL)

        self.assertArgHasType(MapKit.MKMapPointsPerMeterAtLatitude, 0, objc._C_DBL)
        self.assertResultHasType(MapKit.MKMapPointsPerMeterAtLatitude, objc._C_DBL)

        self.assertArgHasType(
            MapKit.MKMetersBetweenMapPoints, 0, MapKit.MKMapPoint.__typestr__
        )
        self.assertArgHasType(
            MapKit.MKMetersBetweenMapPoints, 1, MapKit.MKMapPoint.__typestr__
        )
        self.assertResultHasType(MapKit.MKMetersBetweenMapPoints, objc._C_DBL)

        c = MapKit.MKMapPointMake(1.5, 2.5)
        self.assertIsInstance(c, MapKit.MKMapPoint)
        self.assertEqual(c.x, 1.5)
        self.assertEqual(c.y, 2.5)

        c = MapKit.MKMapSizeMake(1.5, 2.5)
        self.assertIsInstance(c, MapKit.MKMapSize)
        self.assertEqual(c.width, 1.5)
        self.assertEqual(c.height, 2.5)

        c = MapKit.MKMapRectMake(1.5, 2.5, 3.5, 4.5)
        self.assertIsInstance(c, MapKit.MKMapRect)
        self.assertEqual(c.origin, MapKit.MKMapPoint(1.5, 2.5))
        self.assertEqual(c.size, MapKit.MKMapSize(3.5, 4.5))

        self.assertIsInstance(MapKit.MKMapRectGetMinX(c), float)
        self.assertIsInstance(MapKit.MKMapRectGetMinY(c), float)
        self.assertIsInstance(MapKit.MKMapRectGetMidX(c), float)
        self.assertIsInstance(MapKit.MKMapRectGetMidY(c), float)
        self.assertIsInstance(MapKit.MKMapRectGetMaxX(c), float)
        self.assertIsInstance(MapKit.MKMapRectGetMaxY(c), float)
        self.assertIsInstance(MapKit.MKMapRectGetWidth(c), float)
        self.assertIsInstance(MapKit.MKMapRectGetHeight(c), float)

        self.assertIsInstance(
            MapKit.MKMapPointEqualToPoint(MapKit.MKMapPoint(), MapKit.MKMapPoint()),
            bool,
        )
        self.assertIsInstance(
            MapKit.MKMapSizeEqualToSize(MapKit.MKMapSize(), MapKit.MKMapSize()), bool
        )
        self.assertIsInstance(
            MapKit.MKMapRectEqualToRect(MapKit.MKMapRect(), MapKit.MKMapRect()), bool
        )
        self.assertIsInstance(MapKit.MKMapRectIsNull(MapKit.MKMapRect()), bool)
        self.assertIsInstance(MapKit.MKMapRectIsEmpty(MapKit.MKMapRect()), bool)
        self.assertIsInstance(MapKit.MKStringFromMapPoint(MapKit.MKMapPoint()), str)
        self.assertIsInstance(MapKit.MKStringFromMapSize(MapKit.MKMapSize()), str)
        self.assertIsInstance(MapKit.MKStringFromMapRect(MapKit.MKMapRect()), str)
        self.assertIsInstance(
            MapKit.MKMapRectUnion(MapKit.MKMapRect(), MapKit.MKMapRect()),
            MapKit.MKMapRect,
        )
        self.assertIsInstance(
            MapKit.MKMapRectIntersection(MapKit.MKMapRect(), MapKit.MKMapRect()),
            MapKit.MKMapRect,
        )
        self.assertIsInstance(
            MapKit.MKMapRectInset(MapKit.MKMapRect(), 0.0, 0.0), MapKit.MKMapRect
        )
        self.assertIsInstance(
            MapKit.MKMapRectOffset(MapKit.MKMapRect(), 0.0, 0.0), MapKit.MKMapRect
        )

        self.assertArgIsOut(MapKit.MKMapRectDivide, 1)
        self.assertArgIsOut(MapKit.MKMapRectDivide, 2)
        self.assertResultIsBOOL(MapKit.MKMapRectContainsPoint)
        self.assertResultIsBOOL(MapKit.MKMapRectContainsRect)
        self.assertResultIsBOOL(MapKit.MKMapRectIntersectsRect)

        MapKit.MKCoordinateRegionForMapRect  # XXX: Nothing to test beyond existance
        MapKit.MKMapRectRemainder  # XXX: Nothing to test beyond existance

        self.assertResultIsBOOL(MapKit.MKMapRectSpans180thMeridian)



from PyObjCTools.TestSupport import TestCase, min_os_level

import MapKit


class TestMKDistanceFormatter(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(MapKit.MKDistanceFormatterUnitStyle)
        self.assertIsEnumType(MapKit.MKDistanceFormatterUnits)

    @min_os_level("10.9")
    def testConstants(self):
        self.assertEqual(MapKit.MKDistanceFormatterUnitsDefault, 0)
        self.assertEqual(MapKit.MKDistanceFormatterUnitsMetric, 1)
        self.assertEqual(MapKit.MKDistanceFormatterUnitsImperial, 2)
        self.assertEqual(MapKit.MKDistanceFormatterUnitsImperialWithYards, 3)

        self.assertEqual(MapKit.MKDistanceFormatterUnitStyleDefault, 0)
        self.assertEqual(MapKit.MKDistanceFormatterUnitStyleAbbreviated, 1)
        self.assertEqual(MapKit.MKDistanceFormatterUnitStyleFull, 2)



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit


class TestMKMapViewHelper(MapKit.NSObject):
    def mapView_regionWillChangeAnimated_(self, v, a):
        pass

    def mapView_regionDidChangeAnimated_(self, v, a):
        pass

    def mapViewDidFinishRenderingMap_fullyRendered_(self, v, a):
        pass

    def mapView_annotationView_didChangeDragState_fromOldState_(self, v, av, s1, s2):
        pass


class TestMKMapView(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(MapKit.MKOverlayLevel)
        self.assertIsEnumType(MapKit.MKUserTrackingMode)

    @min_os_level("10.9")
    def testConstants(self):
        self.assertEqual(MapKit.MKUserTrackingModeNone, 0)
        self.assertEqual(MapKit.MKUserTrackingModeFollow, 1)
        self.assertEqual(MapKit.MKUserTrackingModeFollowWithHeading, 2)

        self.assertEqual(MapKit.MKOverlayLevelAboveRoads, 0)
        self.assertEqual(MapKit.MKOverlayLevelAboveLabels, 1)

    @min_os_level("10.9")
    def testProtocols(self):
        self.assertIsInstance(
            objc.protocolNamed("MKMapViewDelegate"), objc.formal_protocol
        )

        self.assertArgIsBOOL(TestMKMapViewHelper.mapView_regionWillChangeAnimated_, 1)
        self.assertArgIsBOOL(TestMKMapViewHelper.mapView_regionDidChangeAnimated_, 1)
        self.assertArgIsBOOL(
            TestMKMapViewHelper.mapViewDidFinishRenderingMap_fullyRendered_, 1
        )
        self.assertArgHasType(
            TestMKMapViewHelper.mapView_annotationView_didChangeDragState_fromOldState_,
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestMKMapViewHelper.mapView_annotationView_didChangeDragState_fromOldState_,
            3,
            objc._C_NSUInteger,
        )

    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(MapKit.MKMapView, objc.objc_class)

        self.assertArgIsBOOL(MapKit.MKMapView.setRegion_animated_, 1)
        self.assertArgIsBOOL(MapKit.MKMapView.setCenterCoordinate_animated_, 1)
        self.assertArgIsBOOL(MapKit.MKMapView.setVisibleMapRect_animated_, 1)
        self.assertArgIsBOOL(
            MapKit.MKMapView.setVisibleMapRect_edgePadding_animated_, 2
        )
        self.assertArgIsBOOL(MapKit.MKMapView.setCamera_animated_, 1)

        self.assertArgIsBOOL(MapKit.MKMapView.setZoomEnabled_, 0)
        self.assertArgIsBOOL(MapKit.MKMapView.setScrollEnabled_, 0)
        self.assertArgIsBOOL(MapKit.MKMapView.setRotateEnabled_, 0)
        self.assertArgIsBOOL(MapKit.MKMapView.setPitchEnabled_, 0)

        self.assertResultIsBOOL(MapKit.MKMapView.isZoomEnabled)
        self.assertResultIsBOOL(MapKit.MKMapView.isScrollEnabled)
        self.assertResultIsBOOL(MapKit.MKMapView.isRotateEnabled)
        self.assertResultIsBOOL(MapKit.MKMapView.isPitchEnabled)

        self.assertArgIsBOOL(MapKit.MKMapView.setShowsPointsOfInterest_, 0)
        self.assertArgIsBOOL(MapKit.MKMapView.setShowsBuildings_, 0)
        self.assertArgIsBOOL(MapKit.MKMapView.setShowsUserLocation_, 0)

        self.assertResultIsBOOL(MapKit.MKMapView.showsPointsOfInterest)
        self.assertResultIsBOOL(MapKit.MKMapView.showsBuildings)
        self.assertResultIsBOOL(MapKit.MKMapView.showsUserLocation)

        self.assertResultIsBOOL(MapKit.MKMapView.isUserLocationVisible)
        self.assertArgIsBOOL(MapKit.MKMapView.selectAnnotation_animated_, 1)
        self.assertArgIsBOOL(MapKit.MKMapView.deselectAnnotation_animated_, 1)
        self.assertArgIsBOOL(MapKit.MKMapView.showAnnotations_animated_, 1)

        self.assertResultIsBOOL(MapKit.MKMapView.showsCompass)
        self.assertArgIsBOOL(MapKit.MKMapView.setShowsCompass_, 0)

    @min_os_level("10.10")
    def testClasses10_10(self):
        self.assertResultIsBOOL(MapKit.MKMapView.showsScale)
        self.assertArgIsBOOL(MapKit.MKMapView.setShowsScale_, 0)

    @min_os_level("10.11")
    def testClasses10_11(self):
        self.assertResultIsBOOL(MapKit.MKMapView.showsTraffic)
        self.assertArgIsBOOL(MapKit.MKMapView.setShowsTraffic_, 0)

    @min_os_level("10.15")
    def testClasses10_15(self):
        self.assertResultIsBOOL(MapKit.MKMapView.showsPointsOfInterest)
        self.assertArgIsBOOL(MapKit.MKMapView.setShowsPointsOfInterest_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit


class TestMKMapItem(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(MapKit.MKMapItem, objc.objc_class)

        self.assertResultIsBOOL(MapKit.MKMapItem.isCurrentLocation)
        self.assertResultIsBOOL(MapKit.MKMapItem.openInMapsWithLaunchOptions_)
        self.assertResultIsBOOL(MapKit.MKMapItem.openMapsWithItems_launchOptions_)

    @min_os_level("10.9")
    def testConstants(self):
        self.assertIsInstance(MapKit.MKLaunchOptionsDirectionsModeKey, str)
        self.assertIsInstance(MapKit.MKLaunchOptionsMapTypeKey, str)
        self.assertIsInstance(MapKit.MKLaunchOptionsShowsTrafficKey, str)
        self.assertIsInstance(MapKit.MKLaunchOptionsDirectionsModeDriving, str)
        self.assertIsInstance(MapKit.MKLaunchOptionsDirectionsModeWalking, str)
        self.assertIsInstance(MapKit.MKLaunchOptionsMapCenterKey, str)
        self.assertIsInstance(MapKit.MKLaunchOptionsMapSpanKey, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(MapKit.MKLaunchOptionsCameraKey, str)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(MapKit.MKLaunchOptionsDirectionsModeTransit, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(MapKit.MKLaunchOptionsDirectionsModeDefault, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(MapKit.MKMapItemTypeIdentifier, str)
        self.assertIsInstance(MapKit.MKMapViewDefaultAnnotationViewReuseIdentifier, str)
        self.assertIsInstance(
            MapKit.MKMapViewDefaultClusterAnnotationViewReuseIdentifier, str
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit


class TestMKPolygon(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(MapKit.MKPolygon, objc.objc_class)

        self.assertArgSizeInArg(MapKit.MKPolygon.polygonWithPoints_count_, 0, 1)
        self.assertArgIsIn(MapKit.MKPolygon.polygonWithPoints_count_, 0)

        self.assertArgSizeInArg(
            MapKit.MKPolygon.polygonWithPoints_count_interiorPolygons_, 0, 1
        )
        self.assertArgIsIn(
            MapKit.MKPolygon.polygonWithPoints_count_interiorPolygons_, 0
        )

        self.assertArgSizeInArg(MapKit.MKPolygon.polygonWithCoordinates_count_, 0, 1)
        self.assertArgIsIn(MapKit.MKPolygon.polygonWithCoordinates_count_, 0)

        self.assertArgSizeInArg(
            MapKit.MKPolygon.polygonWithCoordinates_count_interiorPolygons_, 0, 1
        )
        self.assertArgIsIn(
            MapKit.MKPolygon.polygonWithCoordinates_count_interiorPolygons_, 0
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import MapKit


class TestMKOverlayRenderer(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(MapKit.MKOverlayRenderer, objc.objc_class)

        self.assertResultIsBOOL(MapKit.MKOverlayRenderer.canDrawMapRect_zoomScale_)

    @min_os_level("10.9")
    def testFunctions(self):
        MapKit.MKRoadWidthAtZoomScale
